@@ -124,7 +124,7 @@ readOutput<-function(outputfilename="Rprof.out"){
         #Read and prepare output file
 	RprofSamples<-readLines(outputfilename)
         if(length(grep("line profiling",RprofSamples[1]))==0){
-        stop("Line profiling is required. /nPlease run the profiler with line profiling enabled")}
+        stop("Line profiling is required. \nPlease run the profiler with line profiling enabled")}
 	splitCalls<- sapply(RprofSamples[-1],
 	function(X) strsplit(X, split = " "),USE.NAMES=FALSE)
 	#seperated function calls
@@ -155,7 +155,7 @@ readLineDensity<-function(aprofobject=NULL,Memprof=FALSE){
 
   if(!"aprof"%in%class(aprofobject)){
       stop("no aprof object found, check function inputs")}
-  
+
   
   if(Memprof==TRUE){
 
@@ -167,52 +167,83 @@ readLineDensity<-function(aprofobject=NULL,Memprof=FALSE){
     interval <- aprofobject$interval
     TargetFile <- aprofobject$sourcefile
     }
+  ## find all files in the call stack
+  idfiles<-sapply(calls,function(X) length(grep("#File", X))>=1)
+  ## extract files
+  CallFiles <- sapply(calls[idfiles],function(X) X[1])
   
-    if(is.null(TargetFile)){
-      FileNumber<-"1:"
-      warning("sourcefile is null, assuming first file in call stack is the source")
-    } else{
-      unlistedCalls <- unlist(calls)
+  if(is.null(TargetFile)){
+    FileNumber<-"1:"
 
-     TargetFile <- basename(TargetFile)
-     FileNumber<-unlistedCalls[which(unlistedCalls==TargetFile)+1]}
+    warning(paste("sourcefile is null",
+                  " assuming first file in call stack is the source: ",
+                  CallFiles[1],sep=""))
+    
+    if(!exists(CallFiles[1])){stop("source file was not defined and does not seem to exist in the working directory.")}
+    
+  } else{
+    unlistedCalls <- unlist(calls)
 
+    ## add path or only stick to basename?
+    if(sum(unlistedCalls==TargetFile)==0){
+      if(sum(unlistedCalls==basename(TargetFile))>0){
+      TargetFile <- basename(TargetFile)
+    } else {
+       warning(paste("specified source file ", TargetFile,
+                  " is not in the list of files in the profiler output: \n ",
+                  CallFiles,sep=""))
+     }
+    } 
+    
+    FileNumber<-unlistedCalls[which(unlistedCalls==TargetFile)+1]
+    FileCheck<-unlistedCalls[which(unlistedCalls==TargetFile)]
+    
+
+    ## Confirm that call stack corresponds to user supplied source file
+    if(length(FileCheck)==0){
+      warning(paste("Some aprof functions may fail -->",
+                    " user supplied source file ",TargetFile,
+                    " does not seem to correspond to any",
+                    " file in the profiler output.\n",
+                    " Possible causes: \n" ,
+                    "1) Source file was not profiled?\n",
+                    "2) Spelling?\n",sep=""))
+      
+      
+    } 
+    
+  }
+  
   FileNumber <- substr(FileNumber,1,1)
-         
-	cleancalls<-sapply(calls, function(x) 
-	gsub("#File", NA, x))
+  
+  ## remove all file references  
+  cleancalls<-sapply(calls[!idfiles], function(x) 
+                     gsub("#File", NA, x))
+  
+  LineCalls<- sapply(cleancalls,
+                     function(X) X[grep(paste(FileNumber,"#",sep=''),X)],
+                     USE.NAMES=FALSE)
+  
+  Pathways<-unique(sapply(LineCalls, paste,collapse="-"))
 
-	LineCalls<- sapply(cleancalls,
-		function(X) X[grep(paste(FileNumber,"#",sep=''),X)],
-                           USE.NAMES=FALSE)
-		
-	Pathways<-unique(sapply(LineCalls, paste,collapse="-"))
-	# limit only those containing information
-	Pathways<-Pathways[grep("#",Pathways)]
+  ## limit only those containing information
+  Pathways<-Pathways[grep("#",Pathways)]
 
-	FirstRow<-sapply(Pathways,function(X) 
-	strsplit(X, split = "-")[[1]][1],USE.NAMES=FALSE)
-
-	if(length(unique(FirstRow))==1){
-	# remove fist row from calls
-	SimpleCalls<-sapply(LineCalls,
-		function(X) X[-grep(unique(FirstRow),X)])
-	LineDensity<-table(unlist(sapply(SimpleCalls,unique)))    
-	} else {
-	LineDensity<-table(unlist(sapply(LineCalls,unique)))  
-	}
-	names(LineDensity)<-gsub("1#","",names(LineDensity))
-	Line.Numbers<-as.numeric(names(LineDensity))
-	Call.Density<-as.numeric(LineDensity)
-	Time.Density<-Call.Density*interval
-
-	Finallist<-list(Line.Numbers=as.numeric(names(LineDensity)),
-	Call.Density=as.numeric(LineDensity),
-	Time.Density=Call.Density*interval,
-	Total.Calls=sum(as.numeric(LineDensity))+1,
-	Total.Time=sum(Call.Density*interval+interval))
-	
-return(Finallist)
+  filehash <-  paste(FileNumber,"#",sep="")
+  LineDensity<-table(unlist(sapply(LineCalls,unique)))  
+   names(LineDensity)<-gsub(filehash,"",names(LineDensity))
+  Line.Numbers<-as.numeric(names(LineDensity))
+  Call.Density<-as.numeric(LineDensity)
+  Time.Density<-Call.Density*interval
+  
+  Finallist<-list(Line.Numbers=as.numeric(names(LineDensity)),
+                  Call.Density=as.numeric(LineDensity),
+                  Time.Density=Call.Density*interval,
+                  Total.Calls=sum(as.numeric(LineDensity))+1,
+                  Total.Time=sum(Call.Density*interval+interval),
+                  Files=CallFiles)
+  
+  return(Finallist)
 }
 
 #' Generic print method for aprof objects
@@ -243,10 +274,12 @@ aprofobject<-x
     Finallist <- readLineDensity(aprofobject,Memprof=FALSE)
 
 	# Pretty table 
-	CallTable<-cbind(as.character(Finallist$Line.Numbers),
+    CallTable<-cbind(as.character(Finallist$Line.Numbers),
 					Finallist$Call.Density,
 					Finallist$Time.Density)
-	CallTable<-CallTable[order(CallTable[,2]),]
+
+    if(nrow(CallTable)>1) {CallTable<-CallTable[order(CallTable[,2]),]}
+    
         rownames(CallTable)<-NULL
 	dimnames(CallTable)<-list(NULL, c("Line","Call Density",
                                     "Time Density (s)"))
@@ -260,7 +293,10 @@ aprofobject<-x
 			 "Calls\t\t",Finallist$Total.Calls,"\n",
 			 "Time (s)\t",Finallist$Total.Time,
                                    "\t(interval = \t",interval,"(s))\n"))
-	  
+    if(length(Finallist$Files)>1) {
+	  cat("\n Note: multiple files in the profiler output: \n")
+	 print.default(Finallist$Files,print.gap = 2L,quote = FALSE)
+    }
     
 	} else {
          memFinallist <- readLineDensity(aprofobject,Memprof=TRUE)
@@ -627,8 +663,7 @@ profileplot <- function(aprofobject){
  opar<-par("mar","bg")
  maxtimesteps <- max(timesteps)
 
- layoutmat<-matrix(c(rep(c(1,1,1,1,2,2),10)),
-                   byrow=T,ncol=6)
+ layoutmat<-matrix(c(rep(c(1,1,1,1,2,2),10)), byrow=T,ncol=6)
  
  layout(layoutmat)
  
@@ -752,6 +787,7 @@ aprofobject<-object
 	Speedups<-2^c(0:4)
 	SpeedTable<-sapply(Speedups,function(X) AmLaw(P=PropLines,S=X))
 
+        if(is.null(nrow(SpeedTable))) SpeedTable <- matrix(SpeedTable,nrow=1)
 	#Time improvement table 
 	ExecTimeTable<-LineProf$Total.Time/SpeedTable
 	ExecTimeTable<-rbind(ExecTimeTable,LineProf$Total.Time/Speedups)
@@ -887,7 +923,7 @@ targetedSummary<-function(target=NULL,aprofobject=NULL,findParent=FALSE,
                     
           }
           
-        # Sort decending and save as data.frame
+          ## Sort decending and save as data.frame
         CallOrder <- order(CallCounts,decreasing=TRUE)
         CallCounts <- CallCounts[CallOrder]
         parentCalls <- parentCalls[CallOrder]
@@ -909,3 +945,4 @@ targetedSummary<-function(target=NULL,aprofobject=NULL,findParent=FALSE,
                              
 
 }
+ 
